@@ -12,7 +12,7 @@
 # (2) Added rules based paging (Stephen Johnson  Nov 14, 1998)
 # (3) Added checks against Acks and downtime (Stephen Johnson  Mar 17, 1999)
 #
-# $Id: spong-message.pl,v 1.31 2001/06/21 17:05:00 supermathie Exp $
+# $Id: spong-message.pl,v 1.36 2002/05/10 14:43:09 sljohnson Exp $
 
 use lib "@@LIBDIR@@";
 
@@ -502,7 +502,7 @@ RULE:
          add_contact(\@con,$contact);
       }
 
-      if( $match and $rules->{'last_if_match'} == 1 ) {
+      if( $match and $rule->{'last_if_match'} == 1 ) {
          debug("'last_if_match' flag set on rule $c $rule->{'name'}," .
                " exiting rules scan");
          last;
@@ -630,6 +630,8 @@ RULE:
 
                ($subj,$body) = eval_template($person,$f);
 
+               debug("Calling msg function $f for $person");
+
                # Call the message function as referenced by the MSGFUNCS hash
                eval { (&{$MSGFUNCS{$f}}($HUMANS{$person}->{$f},$subj,$body)); };
 
@@ -643,6 +645,8 @@ RULE:
 
             ($subj,$body) = eval_template($person,$func);
 
+            debug("Calling msg function $f for $person");
+
             # Call the message function as referenced by the MSGFUNCS hash
             eval { &{$MSGFUNCS{$func}}($HUMANS{$person}->{$func},$subj,$body); };
 
@@ -705,11 +709,17 @@ sub add_contact {
    &debug("Adding contact " . (ref($contact) eq 'HASH' ? "hash: " .
           $contact->{'rcpt'} : ": " . $contact));
    if ( ref($contact) eq 'HASH') {
-#      foreach $rcpt ( split /[, \t]+/, $contact->{'rcpt'} ) {
-         if ( $HUMANS{ $contact->{'rcpt'} }{'group'} ) {
+         $contact->{'rcpt'} =~ /^([^:]+)(:(.*))?/;
+         my($rcpt,$form) = ($1,$3);
+         if ( $HUMANS{$rcpt}{'group'} ) {
+            &debug("Found group: $rcpt");
             my $groupee;
-            foreach $groupee ( split /[, \t]+/,
-                                $HUMANS{ $contact->{'rcpt'} }{'group'} ) {
+            foreach $groupee ( split /[, \t]+/, $HUMANS{ $rcpt }{'group'} ) {
+               $groupee =~ /^([^:]+)(:(.*))?/;
+               my($groupee,$gform) = ($1,$3);
+               (defined $form and defined $gform and $form ne $gform) && next;
+               if (defined $form) { $groupee="$groupee:$form"; }
+               elsif (defined $gform) { $groupee="$groupee:$gform"; }
                my $newent = {'rcpt' => $groupee};
                if ( defined $contact->{'delay'} )
                   { $newent->{'delay'} = $contact->{'delay'}; }
@@ -723,13 +733,22 @@ sub add_contact {
             push @con, $contact;
          } 
 #      }
-   } elsif ( $HUMANS{$contact}->{'group'} ) {
-      my $groupee;
-      foreach $groupee ( split /[, \t]+/, $HUMANS{$contact}->{'group'} ) {
-         &add_contact(\@con, $groupee);
-      }
    } else {
-      push @con,$contact;
+      $contact =~ /^([^:]+)(:(.*))?/;
+      my($rcpt,$form) = ($1,$3);
+      if ( $HUMANS{$rcpt}->{'group'} ) {
+         my $groupee;
+         foreach $groupee ( split /[, \t]+/, $HUMANS{$rcpt}->{'group'} ) {
+            $groupee =~ /^([^:]+)(:(.*))?/;
+            my($groupee,$gform) = ($1,$3);
+            (defined $form and defined $gform and $form ne $gform) && next;
+            if (defined $form) { $groupee="$groupee:$form"; }
+            elsif (defined $gform) { $groupee="$groupee:$gform"; }
+            &add_contact(\@con, $groupee);
+         }
+      } else {
+         push @con,$contact;
+      }
    }
 }
 
@@ -829,6 +848,10 @@ sub fill_in {
       $tmp = strftime($DATETIMEFMT,localtime($time));
       $str =~ s/!!DATETIME!!/$tmp/g;
    }
+   if ($str =~ /!!DETAILEDHTML!!/) {
+      $tmp = "<PRE>" . $message . "</pre>"; 
+      $str =~ s/!!DETAILEDHTML!!/$tmp/g;
+   }
 
    return $str;
 }
@@ -843,7 +866,11 @@ sub email {
    open( MAIL, "|$SENDMAIL" ) || die "Can't open sendmail: $!";
    print MAIL "To: $recipient\n";
    print MAIL "Subject: $subject\n";
-   print MAIL "Content-Type: text/plain; charset=us-ascii\n";
+   if ( grep(/<HTML>/,$body) ) { 
+      print MAIL "Content-Type: text/html; charset=us-ascii\n";
+   } else {
+      print MAIL "Content-Type: text/plain; charset=us-ascii\n";
+   }
    print MAIL "\n";                                     # Header/Body boundry
    print MAIL "$body\n";
    close( MAIL );
