@@ -6,7 +6,7 @@
 # There are one or more update processes that listen for status updates 
 # from client programs.
 
-# $Id: spong-server.pl,v 1.45 2001/08/16 13:08:29 sljohnson Exp $
+# $Id: spong-server.pl,v 1.50 2002/05/29 02:01:27 sljohnson Exp $
 
 use lib "@@LIBDIR@@";
 
@@ -30,6 +30,13 @@ use Time::Local;
 use Getopt::Long;
 use POSIX qw(:sys_wait_h);
 
+# Check to see if we have the TCP Wrappers library installed
+eval { 
+  require Authen::Libwrap;
+  import Authen::Libwrap qw(hosts_ctl STRING_UNKNOWN);
+};
+if (! $@) { $wrappers = 1;} else { $wrappers = 0; } 
+
 my $numchild;
 $MAX_CHILDREN = 10;
 
@@ -62,7 +69,11 @@ $HOST        =~ tr/A-Z/a-z/;
 
 
 %DATAFUNCS = ();
+%PREDATAFUNCS = ();
+%POSTDATAFUNCS = ();
 &load_data_funcs();
+&load_predata_funcs();
+&load_postdata_funcs();
 &debug("Done loading data plugins");
 
 %PROCS = ();   # All of the procs that we will be monitoring in the main process
@@ -172,7 +183,7 @@ sub listen_for_clients {
    };
 
    # Set up the socket to listen to
-   $sock = IO::Socket::INET->new( Listen   => SOMAXCONN,
+   $sock = IO::Socket::INET->new( Listen   => 1024,
                                   LocalPort => $main::SPONG_CLIENT_PORT,
                                   Proto    => 'tcp',
                                   Timeout  => 30,
@@ -185,9 +196,7 @@ sub listen_for_clients {
    while( 1 ) {
       next unless ( $client = $sock->accept() );
 
-      $paddr = $client->peerhost();
-
-      # &validate_connection( $paddr );  - need to do something here...
+      &validate_connection("spong-client", $client) or next;
 
       # Read all from the client, and disconnect, we process the message next.
 
@@ -222,7 +231,7 @@ sub listen_for_updates {
 
    # Set up the socket to listen to
    my( $sock, $client );
-   $sock = IO::Socket::INET->new( Listen   => SOMAXCONN,
+   $sock = IO::Socket::INET->new( Listen   => 1024,
                                   LocalPort => $main::SPONG_UPDATE_PORT,
                                   Proto    => 'tcp',
                                   Timeout  => 30,
@@ -243,7 +252,7 @@ sub listen_for_updates {
       &debug("Connection from $paddr",6);
       $0 = "spong-server (spong-update) connection from $paddr";
 
-      # &validate_connection( $paddr );  - need to do something here...
+      &validate_connection("spong-update", $client) or next;
 
       # Now fork and allow the kid to process the message
       my $pid = fork();
@@ -316,7 +325,7 @@ sub listen_for_bb_updates {
 
    # Set up the socket to listen to
    my( $sock, $client );
-   $sock = IO::Socket::INET->new( Listen   => SOMAXCONN,
+   $sock = IO::Socket::INET->new( Listen   => 1024,
                                   LocalPort => $main::SPONG_BB_UPDATE_PORT,
                                   Proto    => 'tcp',
                                   Timeout  => 30,
@@ -337,7 +346,7 @@ sub listen_for_bb_updates {
       &debug("Connection from $paddr",6);
       $0 = "spong-server (spong-bb-update) connection from $paddr";
 
-      # &validate_connection( $paddr );  - need to do something here...
+      &validate_connection("spong-bb-update", $client) or next;
 
 
       # Now fork and allow the kid to process the message
@@ -406,7 +415,7 @@ sub listen_for_queries {
    # Set up the socket to listen to
    my( $localTimeOut ) = 15 ;
    my( $sock, $client );
-   $sock = IO::Socket::INET->new( Listen   => SOMAXCONN,
+   $sock = IO::Socket::INET->new( Listen   => 1024,
                                   LocalPort => $main::SPONG_QUERY_PORT,
                                   Proto    => 'tcp',
                                   Timeout  => 30,
@@ -437,9 +446,9 @@ sub listen_for_queries {
               }
             }
           }
-          $next_update += $localTimeOut * 60;
+          $next_update = time() + $localTimeOut * 60;
        } else {
-          &debug(time() . " < $next_update");
+          &debug(time() . " < $next_update", 9);
       }
 
 
@@ -449,7 +458,7 @@ sub listen_for_queries {
       &debug("[$$] update: Connection from $paddr",6);
       $0 = "spong-server (query) connection from $paddr";
 
-      # &validate_connection( $paddr );  - need to do something here...
+      &validate_connection("spong-query", $client) or next;
 
 
       # Now fork and allow the kid to process the message
@@ -608,17 +617,36 @@ sub save_status {
       if ($header =~ /^timestamp (\d+) (\d+)/) { $start = $1; }
       close FILE;
    }
+   $duration = $time - $start;
+
+   # Build message hash for data plugin calls
+   my $msg = { 'header'   => $header,
+               'message'  => $message,
+               'cmd'      => $cmd,
+               'host'     => $host,
+               'service'  => $service,
+               'color'    => $color,
+               'time'     => $time,
+               'duration' => $duration,
+               'summary'  => $sum,
+             };
+
+   # Call of the data function modules in the registry
+   foreach my $df (sort(keys %PREDATAFUNCS)) {
+      &debug("[$$] Running pre data function $df",4);
+      &{$PREDATAFUNCS{$df}}( $msg );
+   }
+
+   # If a Predata plugin says to drop the message, drop it
+   if ( $msg->{'drop_msg'} ) {
+      debug( "[$$] signal from pre data func module, dropping message",5);
+      return;
+   }
 
    # Before we change the state of the database, call message_user()...
-
-   $duration = $time - $start;
    &message_user( "status", $host, $service, $color, $time, $sum, $duration,
                   $message );
 
-   # Make sure that this service is registered in our master service list.
-
-   $main::SERVICES{$service} = 1;
-
    # Save the status information to the database, if there is a change in
    # color, then update the history table and state file as well.
 
@@ -647,7 +675,13 @@ sub save_status {
                $expire );
 
    # Call of the data function modules in the registry
-   foreach my $df (keys %DATAFUNCS) {
+   foreach my $df (sort(keys %POSTFUNCS)) {
+      &debug("[$$] Running post data function $df",4);
+      &{$POSTFUNCS{$df}}( $msg );
+   }
+
+   # Call of the data function modules in the registry
+   foreach my $df (sort(keys %DATAFUNCS)) {
       &debug("[$$] Running data function $df",4);
 
       &{$DATAFUNCS{$df}}($host, $service, $color, $start, $time, $sum, 
@@ -840,11 +874,33 @@ sub save_ack {
       &error( "save_ack: invalid header [$header]" ); return;
    }
 
-   return if $main::HOSTS{$host} eq "";
+   if ($main::HOSTS{$host} eq "") {
+      &error("save_ack: undefined host [$host]");
+      return;
+   }
 
-   # Before we change the state of the database, call message_user()...
+   # Build message hash for the ack
+   my $msg = { 'header'     => $header,
+               'message'    => $message,
+               'cmd'        => $cmd,
+               'host'       => $host,
+               'service'    => $service,
+               'start_time' => $now,
+               'end_time'   => $time,
+               'user'       => $user,
+            };
 
-#   &message_user( "ack", $host, $service, $color, $summ);
+   # Call of the data function modules in the registry
+   foreach my $df (sort(keys %PREDATAFUNCS)) {
+      &debug("[$$] Running pre data function $df",4);
+      &{$PREDATAFUNCS{$df}}( $msg );
+   }
+
+   # If a Predata plugin says to drop the message, drop it
+   if ( $msg->{'drop_msg'} ) {
+      debug( "[$$] signal from pre data func module, dropping message",5);
+      return;
+   }
 
    # Clean up expired acknowledgments.
 
@@ -870,10 +926,16 @@ sub save_ack {
      # Save the status update information
      $untiltime = POSIX::strftime( "%H:%M, %D", localtime($time) );
      $data = "timestamp $now $time\ncolor blue\n$now $user ack $service Until $untiltime \nack:  $service by $user \n$message\n";
-     &save_data( ">>", "$SpongConf::SPONGDB/$host/history/status/$now-$service", $data);
+     &save_data( ">>", "$SPONGDB/$host/history/status/$now-$service", $data);
 
    }
 
+   # Call of the data function modules in the registry
+   foreach my $df (sort(keys %POSTDATAFUNCS)) {
+      &debug("[$$] Running post data function $df",4);
+      &{$POSTDATAFUNCS{$df}}( $msg );
+   }
+
 }
 
 # Take the incoming acknowledgment message, run it through some error checking
@@ -896,6 +958,27 @@ sub del_ack {
    if( $main::HOSTS{$host} eq "" ) {
       &error( "del_ack: invalid host [$host]" ); return; }
 
+   # Build a hash of the del_ack message for data plugin runs
+   my $msg = { 'header'   => $header,
+               'message'  => $message,
+               'cmd'      => $cmd,
+               'host'     => $host,
+               'service'  => $service,
+               'end_time' => $end,
+             };
+
+   # Call of the data function modules in the registry
+   foreach my $df (keys %PREDATAFUNCS) {
+      &debug("[$$] Running pre data function $df",4);
+      &{$PREDATAFUNCS{$df}}( $msg );
+   }
+
+   # If a Predata plugin says to drop the message, drop it
+   if ( $msg->{'drop_msg'} ) {
+      debug( "[$$] signal from pre data func module, dropping message",5);
+      return;
+   }
+
    # Remove the acknowledgement that the user specified.
 
    opendir( DIR, "$main::SPONGDB/$host/acks" );
@@ -908,6 +991,13 @@ sub del_ack {
    # I don't updated the history when an ack is deleted, I suppose I should
    # but it makes things a little messing (since an update is just a delete
    # and a re-add).
+
+   # Call of the data function modules in the registry
+   foreach my $df (sort(keys %POSTDATAFUNCS)) {
+      &debug("[$$] Running post data function $df",4);
+      &{$POSTDATAFUNCS{$df}}( $msg );
+   }
+
 }
 
 # ===========================================================================
@@ -1451,14 +1541,14 @@ sub chld_handler {
 sub reaper {
     my $kid;
 
-    $kid = wait(); 
-    $main::numchild--;
-    &debug("reaped kid $kid, # of child = $main::numchild");
+#    $kid = wait(); 
+#    $main::numchild--;
+#    &debug("reaped kid $kid, # of child = $main::numchild");
 
-#   while( ($kid = waitpid(-1,&WNOHANG)) > 0 )  {
-#      $main::numchild--;
-#     &debug("reaped kid $kid, # of child = $main::numchild");
-#   } 
+   while( ($kid = waitpid(-1,&WNOHANG)) > 0 )  {
+      $main::numchild--;
+     &debug("reaped kid $kid");
+   } 
 
    # Reinstate signal handler in cae system uses sysV libs
    $SIG{'CHLD'} = \&reaper;
@@ -1480,9 +1570,86 @@ sub load_data_funcs {
       $file =~ /data_(.*)$/;  $base = $1;
       &debug("Loading data function $base", 3 );
       eval { require "$datafunc_path/$file"; };
-      if ( $@ ) { &error("Could not load messaging function $base: $@"); }
+      if ( $@ ) { &error("Could not load data function $base: $@"); }
    }
 
 }
 
+# ---------------------------------------------------------------------------
+# Load all of the data functions into the POSTDATAFUNCS registry
+# ---------------------------------------------------------------------------
 
+sub load_postdata_funcs {
+   my($file,@files);
+
+   if ( ! -d $datafunc_path) { return; }
+   opendir(MSG,$datafunc_path) or die "Could not opendir $datafunc_path: $!";
+   @files = grep { /^post_/ } readdir(MSG);
+   closedir(MSG);
+
+   foreach $file (@files) {
+      $file =~ /post_(.*)$/;  $base = $1;
+      &debug("Loading post data function $base", 3 );
+      eval { require "$datafunc_path/$file"; };
+      if ( $@ ) { &error("Could not load post data function $base: $@"); }
+   }
+
+}
+
+# ---------------------------------------------------------------------------
+# Load all of the predata functions into the PREDATAFUNCS registry
+# ---------------------------------------------------------------------------
+
+sub load_predata_funcs {
+   my($file,@files);
+
+   if ( ! -d $datafunc_path) { return; }
+   opendir(MSG,$datafunc_path) or die "Could not opendir $datafunc_path: $!";
+   @files = grep { /^pre_/ } readdir(MSG);
+   closedir(MSG);
+
+   foreach $file (@files) {
+      $file =~ /pre_(.*)$/;  $base = $1;
+      &debug("Loading pre data function $base", 3 );
+      eval { require "$datafunc_path/$file"; };
+      if ( $@ ) { &error("Could not load pre data function $base: $@"); }
+   }
+
+}
+
+# ---------------------------------------------------------------------------
+# Validate an incoming connection
+# ---------------------------------------------------------------------------
+
+sub validate_connection {
+    my ($type, $client) = @_;
+
+    # If TCP Wrappers not found, every connection is valid
+    if ( ! $wrappers ) { return 1; }
+
+    my $client_ip = $client->peerhost();
+    my $client_addr = $client->peeraddr();
+
+    my $client_name = gethostbyaddr($client_addr, AF_INET);
+    unless ($client_name) {
+	&error("gethostbyaddr for $client_ip failed - denying access\n");
+	return 0;
+    }
+
+# Do we need to do this lookup, or will hosts_ctl() take care of it
+# for us?
+#    my @addr = gethostbyname($client_name)
+#	or &error("gethostbyname for $client_name failed\n");
+#    unless (grep { $client_addr == $_ } @addr[4..$#addr]) {
+#	&error("Host name does not match IP address: denying access\n");
+#	return 0;
+#    }
+
+    unless (hosts_ctl($type, $client_name, $client_ip, STRING_UNKNOWN)) {
+	&error("Denying $type access to $client_name [$client_ip]\n");
+	return 0;
+    }
+
+    &debug("Allowing $type access to $client_name [$client_ip]\n");
+    return 1;
+}
